/*
 *   Copyright (C) 2012 Yury P. Fedorchenko
 *   yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
#include "chboard.h"
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>

#include <fstream>
#include <sstream>
#ifndef USE_FLTK
#include <inkview.h>
#endif

const char* chgame::RNOTATION="b8d8f8h8a7c7e7g7b6d6f6h6a5c5e5g5b4d4f4h4a3c3e3g3b2d2f2h2a1c1e1g1\0";
const char *CheckersBoard::skill_v[] = {"Beginner","Novice","Average","Good","Expert","Master",
                          "M6","M7", NULL };


void CheckersBoard::save_games(){
  if(games.empty())return;
#ifdef USE_FLTK
  const char* hm = getenv("HOME");
  std::string path=hm;
  path+=SAVE_PATH;
#else
  std::string path=SAVE_PATH;
#endif
  size_t sp=0;
  while(sp!=path.npos){
    sp=path.find_first_of("/",sp+1);
    std::string np=path.substr(0,sp);
    std::cerr<<"process:"<<np<<std::endl;
    if(access(np.c_str(),F_OK)!=0){
      if(mkdir(np.c_str(),S_IRWXU)==-1){
        perror(np.c_str());
        return;
      }
    }
  }
  path+="/"+curgame.date+".pdn";
  std::ofstream out(path.c_str(),std::ios_base::ate);
  if(!out.fail()){
    for(size_t i=0;i<games.size();++i)if(!games[i].loaded)out<<games[i]<<std::endl;
  }else{
    std::cerr<<path<<std::endl;
  }
}
static void trim_string(std::string& str,const std::string stt="\""){
  size_t ps=str.find_last_of(stt);
  if(ps!=str.npos){
    str=str.substr(0,ps);
  }
}
//#include<map>
//std::map<int,bool> fmap;
bool CheckersBoard::load_games(const char*fname){
  std::ifstream gmf(fname);
  bool process_moves(false),gm_begin(false);
  chgame gm;
  gm.loaded=true;
  while(!gmf.fail() && !gmf.eof()){
    std::string line;
    std::getline(gmf,line);
    if(!line.empty()){
      if(line[0]=='['){
        if(process_moves){
          games.push_back(gm);
          curgame=gm;
          gm = chgame();
          gm.loaded=true;
          process_moves=gm_begin=false;
        }
        if(line.compare(0,6,"[Event")==0){
          gm.event=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.event);
        }
        else if(line.compare(0,5,"[Site")==0){
          gm.site=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.site);
        }
        else if(line.compare(0,5,"[Date")==0){
          gm.date=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.date);
        }
        else if(line.compare(0,6,"[Round")==0){
          std::string tmp=line.substr(line.find_first_of("\"")+1);
          trim_string(tmp);
          gm.round=atoi(tmp.c_str());
        }
        else if(line.compare(0,9,"[GameType")==0){
          std::string tmp=line.substr(line.find_first_of("\"")+1);
          trim_string(tmp);
          gm.gametype=atoi(tmp.c_str());
          
        }
        else if(line.compare(0,6,"[White")==0){
          gm.white=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.white);
        }
        else if(line.compare(0,6,"[Black")==0){
          gm.black=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.black);
        }
        else if(line.compare(0,7,"[Result")==0){
          gm.result=line.substr(line.find_first_of("\"")+1);
          trim_string(gm.result);
          gm_begin=true;
        }
        else std::cerr<<"Find unknown:"<<line<<std::endl;
      }
      else if(gm_begin){
        std::stringstream moves(line);
        while(!moves.fail() && !moves.eof()){
          std::string mv;
          moves>>mv;
          if(mv==gm.result){gm_begin=false;break;}
          chmove move;
          std::cerr<<"mv:"<<mv<<std::endl;
          size_t sep=mv.find_first_of("-:x");
          if(sep != mv.npos){
            char sp=mv[sep];
            if(sp=='x' || sp==':')move.capt=true;
            else move.capt=false;
            //if(sp=='x' && gm.gametype!=Checkers::ENGLISH){
            //  gm.gametype=Checkers::ENGLISH;gm.is_white=false;
            //}
            //mv.replace(sp,' ');
            //std::cerr<<"mv ar:"<<mv<<":"<<std::endl;
            std::stringstream mvs(mv);
            while(!mvs.fail() && !mvs.eof()){
              std::string pos;
              std::getline(mvs,pos,sp);
              std::cerr<<"pos:"<<pos<<":"<<std::endl;
              chmove::pos ps;
              if(isdigit(pos[0])){
                //chmessage("Unsupported pdn file!");
                return false;
                if(gm.gametype!=Checkers::ENGLISH){
                  gm.gametype=Checkers::ENGLISH;
                }
                ps.pos=atoi(pos.c_str());
                //fmap[ps.pos]=true;
                //std::cerr<<"atoi:"<<pos<<":"<<ps.pos<<std::endl;
              }else{
                pos[0]=tolower(pos[0]);
                ps.pos=gm.rn.find(pos)/2;
              }
              move.path.push_back(ps);
            }
            gm.writeMove(std::cerr,move);
            std::cerr<<"awm:"<<":"<<std::endl;
            gm.addMove(move);
            process_moves=true;
          } else std::cerr<<"notmove:"<<mv<<std::endl;
        }
      }
    }
  }
  if(process_moves){
    std::cerr<<gm<<std::endl;
    games.push_back(gm);
    curgame=gm;
  }
  return true;
  //for(std::map<int,bool>::iterator it=fmap.begin();it!=fmap.end();++it)std::cerr<<it->first<<std::endl;
}
