/*
 *   Copyright (C) 2009-2012 Yury P. Fedorchenko
 *   yuryfdr@users.sf.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Library General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor Boston, MA 02110-1301,  USA
 */
#ifndef __checkers_board_h__
#define __checkers_board_h__

#include "rcheckers.h"
#include "echeckers.h"


class CheckersBoard {
public:
  static const char *skill_v[];
  Checkers* game;
  bool game_over;
  int  winner;
  // preferences
  int cp;
  Checkers::GameRules rules;
  bool is_white;
  Checkers::GameSkill skill;
  //
  bool ai2,go1;
  int curmove;
  chgame curgame;
  std::vector<chgame> games;
protected:  
  bool check_win(bool human_move){
    if(game_over) return true;
    game_over = true;
    bool player_can = game->checkMove1() || game->checkCapture1();
    bool opp_can =    game->checkMove2() || game->checkCapture2();
    // AI win
    if(human_move && !player_can && opp_can){
      winner=1;
      curgame.result=(is_white)?"1-0":"0-1";
      if(!curgame.loaded)games.push_back(curgame);
      return game_over;
    }
    // Human win
    if(!human_move && player_can && !opp_can){
      winner=2;
      curgame.result=(is_white)?"0-1":"1-0";
      if(!curgame.loaded)games.push_back(curgame);
      return game_over;
    }
    // Nobody cam move
    if(!player_can && !opp_can) {
      winner=3;
      curgame.result="1/2-1/2";
      if(!curgame.loaded)games.push_back(curgame);
      return game_over;
    }
    game_over = false;
    return game_over;
  };
  virtual void set_checkers() = 0;
public:
  // create new game
  void new_game(const chgame& ldgame){
    //if(game) delete game;
    rules=(Checkers::GameRules)ldgame.gametype;
    curgame=ldgame;
    curgame.loaded=true;
    curmove=0;
    if(Checkers::RUSSIAN == rules )game=new RCheckers();
    else game=new ECheckers();
    is_white=curgame.is_white;
    set_checkers();
    game_over = false;
    winner = 0;
  }
  void continue_game(bool white){
    game->setSkill(skill);
    is_white=white;
    curgame.moves.resize(curmove);
    curgame.loaded=false;
    chmove mv;
    if(curmove%2==0 && !is_white){
      game->go2(mv);curgame.moves.push_back(mv);
    }
    if(curmove%2!=0 && is_white){
      game->go2(mv);curgame.moves.push_back(mv);
    }
    go1=true;
  }
  void new_game(bool h2)
  {
    if(game) delete game;
    if(Checkers::RUSSIAN == rules )game=new RCheckers();
    else game=new ECheckers();
    game->setSkill(skill);

    static char dt[128];
    time_t tim=time(NULL);
    struct tm* tmp = localtime(&tim);
    strftime(dt,128,"%Y.%m.%d",tmp);
    curgame = chgame(dt,games.size()+1,game->type());
    curmove=0;
    curgame.is_white=is_white;
    if(Checkers::RUSSIAN == rules ){
      if(is_white)curgame.white="Human";
    }else{
      if(is_white)curgame.black="Human";
    }

    set_checkers();
    game_over = false;
    winner = 0;
    ai2=h2;
    
    chmove move;
    if(!ai2){
      if(!is_white){game->go2(move);curgame.moves.push_back(move);curmove++;go1=true;}
      else go1=true;
    }else go1=is_white;
    set_checkers();
  }
  
  CheckersBoard():game(NULL),game_over(true),cp(-1),rules(Checkers::RUSSIAN),
    is_white(true),skill(Checkers::BEGINNER){
    game=new RCheckers();
  }
  virtual ~CheckersBoard(){}
  void save_games();
  bool load_games(const char* fname);
};
#ifdef USE_FLTK
#define SAVE_PATH "/.local/share/yacheckers"
#else
#define SAVE_PATH FLASHDIR "/system/share/yacheckers"
#endif

inline std::ostream& operator<<(std::ostream& out,const chmove& mv){
  //out<<"from:"<<mv.from<<" to:"<<mv.to<<" ln:"<<mv.path.size()<<" cap:"<<mv.capt<<'\n';
  for(size_t i=0;i<mv.path.size();++i)out<<"\t"<<i<<'\t'<<mv.path[i].pos;
  return out;
};
inline std::ostream& operator<<(std::ostream& out,const chgame& gm){
  out<<"[Event \""<<gm.event<<"\"]\n"
     <<"[Site \""<<gm.site<<"\"]\n"
     <<"[Date \""<<gm.date<<"\"]\n"
     <<"[Round \""<<gm.round<<"\"]\n"
     <<"[GameType \""<<gm.gametype<<"\"]\n"
     <<"[White \""<<gm.white<<"\"]\n"
     <<"[Black \""<<gm.black<<"\"]\n"
     <<"[Result \""<<gm.result<<"\"]\n\n";

  for(size_t j=0;j<gm.moves.size();++j){
    if(j%2==0)out<<(j/2+1)<<". ";
    gm.writeMove(out,gm.moves[j]);
  }
  out<<gm.result<<"\n";
  return out;
};
inline std::ostream& chgame::writeMove(std::ostream& out,const chmove& mv)const{
  size_t i=0;
  /*if(gametype==Checkers::ENGLISH){
    if(!is_white){
      for(;i<mv.path.size()-1;++i)out<<mv.path[i].pos<<(mv.capt?"x":"-");
      out<<mv.path[i].pos<<" ";
    }else{
      for(;i<mv.path.size()-1;++i)out<<(31-mv.path[i].pos)<<(mv.capt?"x":"-");
      out<<(31-mv.path[i].pos)<<" ";
    }
  }else*/
  //std::cerr<<"wm:"<<mv.path.size()<<std::endl;
  
  {
    if( is_white /*(is_white && gametype==Checkers::RUSSIAN) || (!is_white && gametype==Checkers::ENGLISH)*/ ){
      for(;i<mv.path.size()-1;++i)out<<rn.substr(2*mv.path[i].pos,2)<<(mv.capt?":":"-");
      out<<rn.substr(2*mv.path[i].pos,2)<<" ";
    }else{
      for(;i<mv.path.size()-1;++i)out<<rn.substr(2*(31-mv.path[i].pos),2)<<(mv.capt?":":"-");
      out<<rn.substr(2*(31-mv.path[i].pos),2)<<" ";
    }
  }
  return out;
};

#endif // __checkers_board_h__

